package gps;

import gps.datatypes.Record;
import gps.exceptions.EmptyBufferException;
import gps.exceptions.ParseException;
import gps.exceptions.UnsupportedTypeException;
//import gps.log.Logger;
import gps.parser.Parser;
import java.text.NumberFormat;


import java.io.IOException;
import java.io.InputStreamReader;

import javax.microedition.io.Connector;
import javax.microedition.io.StreamConnection;

/**
 * Main class for communication with GPS receiver. Use this class to access GPS
 * receiver from other classes. 
 * 
 * @author edited by Henrik Davidsen
 */
public class GPS implements Runnable {
	/**
	 * Time in ms to wait until resume to receive.
	 */
	
	public String lattitude;
	public String lattitudeDirection;
	public String longitude;
	public String longitudeDirection;
	public String groundSpeed; 
	public boolean warning = true;
	public String lat;
	public String lon;
	
	
	public boolean N = false;
	public boolean E = false;
	
	
	
	
	
	
	
	
	private static final long BREAK = 2000;

	/**
	 * Character which indicates end of line in data send by bluetooth device.
	 */
	private static final int LINE_DELIMITER = 13;

	/**
	 * Buffers parsed record.
	 */
	private RecordBuffer buffer;

	/**
	 * Connection to bluetooth device.
	 */
	private StreamConnection connection;

	/**
	 * Used to show messages.
	 */
	//private Logger logger;

	/**
	 * Input reader to read data send by bluetooth device.
	 */
	private InputStreamReader reader;

	/**
	 * Receiving happens in separate thread.
	 */
	private Thread runner;

	/**
	 * URL used to connect to bluetooth device.
	 */
	private String url;

	/**
	 * Creates new receiver. Does not start automatically, use start() instead.
	 * 
	 * @param logger
	 *            Used to show messages.
	 * @param url
	 *            URL of bluetooth device to connect to.
	 */
	public GPS(String url) {
		//this.logger = logger;
		this.url = url;

		this.buffer = new RecordBuffer();
	}

	/**
	 * Establishes a bluetooth serial connection (specified in GPS_BT_URL) and
	 * opens an input stream.
	 * 
	 * @throws IOException
	 *             If error occurs while establishing bluetooth connection or
	 *             opening input stream
	 */
	private synchronized void connect() throws IOException {
		connection = (StreamConnection) Connector.open(url, Connector.READ);
		reader = new InputStreamReader(connection.openInputStream());
	}

	/**
	 * Closes input stream and bluetooth connection as well as sets the
	 * corresponding objects to null.
	 */
	private synchronized void disconnect() {
		try {
			if (reader != null)
				reader.close();
			if (connection != null)
				connection.close();
		} catch (IOException e) {
			// Ignore.
		}
		reader = null;
		connection = null;
	}

	/**
	 * 
	 * @return The currently stored record.
	 * @throws EmptyBufferException
	 *             If no record has been stored yet.
	 */
	public Record getRecord() throws EmptyBufferException {
		return buffer.getRecord();
	}

	/**
	 * @return True, if connected and input stream opened
	 */
	public synchronized boolean isConnected() {
		return connection != null && reader != null;
	}
	
	
	
	public void plotData(){
		
		  try
			{
			if(N && E){
			System.out.println("Opens map location in google maps: ");
			String command = "c:\\programfiler\\Internet Explorer\\iexplore.exe http://maps.google.com/maps?q="+lat+" "+lon;
			Runtime runtime = Runtime.getRuntime();
			Process process = runtime.exec(command);
			if(process.waitFor() == 0){
			 process.destroy();
			 System.out.println("Iexplorer was terminated by user");
			}
			}
			}
			catch(Exception exc){

			}
		
		
	}
	
	
	
	
	public String getGPSdata1(){
		
		return lattitude;
		}
	public String getGPSdata2(){
		return lattitudeDirection;
	}
	
	public String getGPSdata3(){
		
		return longitude;
	}
	
	public String getGPSdata4(){
		return longitudeDirection;
		
	}
	public String getGPSdata5(){
		
		return groundSpeed;
	}
	
	public boolean getWarning(){
		
		return warning;
	}
	
	
	
	

	/**
	 * Reads in records sent by the GPS receiver. When a supported record has
	 * been received pauses for specified amount of time. Continues on I/O
	 * errors.
	 */
	public void run() {
		
		System.out.println("thread of GPS is running");
		// Marker that indicates start of one cycle of records.
		int startType = Parser.TYPE_START;
		// Type of currently parsed record.
		int type = Parser.TYPE_NA;
		// Storage.
		Record record = new Record();

		while (Thread.currentThread() == runner) {
			
			
			try {
				// If not connected (e.g. because of explicit disconnect before
				// waiting) try to connect again.
				if (!isConnected())
					connect();

				String output = new String();

				// Read one line and try to parse it. If successfull put parsed
				// record in buffer.
				int input;
				while ((input = reader.read()) != LINE_DELIMITER)
					output += (char) input;
				// Remove last character (10 in ASCII)
				output = output.substring(1, output.length() - 1);

				try {
					
					
					type = Parser.parse(output, record);
					buffer.putRecord(record);
					System.out.println("dateTimeOfFix:" +record.dateTimeOfFix);
					System.out.println("warning:" +record.warning);
					System.out.println("lattitude:" +record.lattitude);
					System.out.println("lattitudeDirection:" +record.lattitudeDirection);
					System.out.println("longitude:" +record.longitude);
					System.out.println("longitudeDirection:" +record.longitudeDirection);
					System.out.println("groundSpeed:" +record.groundSpeed);
					System.out.println("courseMadeGood:" +record.courseMadeGood);
					System.out.println("magneticVariation:" +record.magneticVariation);
					
					lattitude = record.lattitude.toString();
					lattitudeDirection = record.lattitudeDirection.toString();
					longitude = record.longitude.toString();
					longitudeDirection = record.longitudeDirection.toString();
					groundSpeed = record.groundSpeed.toString();
					warning = record.warning;
					
					NumberFormat nf = NumberFormat.getNumberInstance() ;
					nf.setGroupingUsed(false) ;     // don't group by threes
				    nf.setMaximumFractionDigits(2) ;
				    nf.setMinimumFractionDigits(2) ;
				    nf.setMaximumIntegerDigits(2);
				    nf.setMinimumIntegerDigits(2);
					
					if(lattitude.length()>2 && longitude.length()>2){
					String gpsData1b = lattitude;
					String gpsData2b = longitude;
					  
					float decimalN= Float.parseFloat(gpsData1b.substring(5,9));
					float decimalE= Float.parseFloat(gpsData2b.substring(6,10));
					String convN = "";
					String convE = "";
					
					//Her stod egentlig deklareringen av boolean til false N og E
					
					
					
					if(!lattitude.equals("0000.0000"))
						N = true;
					
					if(!longitude.equals("00000.0000"))
					    E = true;
					
					if(N){
						decimalN = (decimalN/10000)*60;
						convN = nf.format(decimalN);
						convN = convN.substring(0,2)+"."+convN.substring(3,5);
					}
					
					if(E){
						decimalE = (decimalE/10000)*60;
						convE = nf.format(decimalE);
						convE = convE.substring(0,2)+"."+convE.substring(3,5);
					}
					
					System.out.println("decimalN: "+decimalN);
					System.out.println("decimalE: "+decimalE);
					System.out.println("Converted N: "+convN);
					System.out.println("Converted E: "+convE);
					
					  lat = gpsData1b.substring(0,2)+" "+gpsData1b.substring(2,4)+"' "+convN+"\""; 
					  lon = gpsData2b.substring(0,3)+" "+gpsData2b.substring(3,5)+"' "+convE+"\""; 
					  
					  System.out.println("Latitude N:  " + lat);
					  System.out.println("Longitude E: " + lon);
					  System.out.println("N " + lat + "E " + lon);
					  
					
					  
					}

				
				}// her begynner ny kode, alts ikke map orientert
				// Is being thrown when a record is parsed that has an
				// unsupported type.
				catch (UnsupportedTypeException ue) {
					// Type is not supported.
					System.out.println("ue exception e" +ue);
					type = Parser.TYPE_NA;
				}
							
				// Error while parsing (supported) record.
				catch (ParseException pe) {
					//logger.appendString("Parse exception: " + pe.getMessage());
					pe.printStackTrace();
					System.out.println("pe exception e" +pe);		
				
				}

				// Examine only if valid type was parsed.
				if (type != Parser.TYPE_NA) {
					// End of cycle: wait.
					if (type == startType) {
						disconnect();
						// Reset start marker.
						startType = Parser.TYPE_START;
						// Reset startType and storage.
						record = new Record();
						try {
							Thread.sleep(BREAK);
						} catch (InterruptedException e) {
							//logger.appendString(e.getMessage());
						}
					}
					// Set start marker.
					else if (startType == Parser.TYPE_START) {
						startType = type;
					}
				}
			}
			// Most severe type of exception. Either thrown while connecting or
			// while reading. Wait some time before continuing.
			catch (IOException ie) {
				try {
					Thread.sleep(BREAK);
				} catch (InterruptedException e) {
					//logger.appendString("IO exception: " + e.getMessage());
					System.out.println("IO exception e" +ie);
				
				}
				ie.printStackTrace();
			}
		}
	}

	/**
	 * Starts receving of data (if not yet started).
	 *  
	 */
	public void start() {
		if (runner == null) {
			runner = new Thread(this);
			runner.start();
		}
	}

	/**
	 * Stops receiving of data and disconnects from bluetooth device.
	 *  
	 */
	public void stop() {
		runner = null;
		disconnect();
	}
}

